%TactileRecorder v1.0
%John Ulmen
%3/21/12
%This program displays and records data returned through a serial port from
%the Stanford Tactile Sensor.  The appropriate serial port must be 
%specified in code below.  Downsampling and averaging are possible by 
%setting appropriate variables and flags.

clear all
close all
error_reported=false;

%close down any open com ports
if(size(instrfind)~=[0 0])
    disp('Lingering COM port open... closing.'); 
    fclose(instrfind);
    delete(instrfind);
end

%request com number from user
disp(' ')
disp('Please input serial port label string in single quotes')
disp('i.e. Windows ''COM12'', Linux ''/dev/ttyUSB0'',')
com_name = input ('MAC OS X ''/dev/tty.usbserial-A5VAFC9U''): ');
if (ischar(com_name)&&(~error_reported))
    disp([com_name ' entered, opening port...'])
else
    disp('Invalid entry, run program again')
    error_reported=true;
end

%create com object and enter display run time
if (~error_reported)
    %Open the serial port and create com object
    com = serial(com_name, 'BaudRate', 115200,'InputBufferSize',65536);
    fopen(com);
    disp('Serial port open');
    %enter display run time
    display_time = input('Please input time in seconds to run display: ');
    if (isnumeric(display_time)&&(display_time>=0))
        disp([num2str(display_time) ' second display selected']);
    else
        disp('Invalid entry, run program again')
        error_reported=true;
    end
end

%if no errors have occurred during user entry, begin display
if (~error_reported)
    %Definitions and settings
    num_sensors=12; %number of pads on sensor
    packet_size=28; %data packet length, num_sensor*2+4
    record_time=display_time; %time to run display in seconds
    downsample_count=1;   %store only 1 in every downsample_count samples 
    averaging_enabled=0;   %set to 1 to average previous samples for downsample storage
    show_sample_count=0;    %set to 1 to display the sample count just stored in the command window
        %(useful for tracking whether program is running when plotting is disabled)
    plot_yes=0; %set to 1 for marker plotting (exclusive to surface plotting)
    plot_surf_yes=1;    %set to 1 for surf plotting (exclusive to marker plotting, choose one only)
    data_history_length=max(10,ceil(record_time*110/downsample_count)); 
        %number of data points to allocate for storage with a minimum of 10
    sensor_data_history=zeros(data_history_length,num_sensors);
    smooth_length=1; %must be less than angle_history_length and a positive int
    sensor_data=zeros(1,num_sensors);
    initial_sensor_data=zeros(1,num_sensors);
    downsample_data=zeros(1,num_sensors);

    InitialScaling=2000;
    max_surf=InitialScaling;
    scalingtimer=0;
    surf_smooth_length=1;
    surf_history=zeros(surf_smooth_length,num_sensors);

    if plot_yes==1
      plot_h1=plot(0,0);
    end

    %Ask the PIC to stream packets of data
    fwrite(com,[2,128,3]);
    %wait a little bit
    pause(0.05);
    %ask again
    %fwrite(com,[2,128,3]);

    %start recording timer
    tic;
    %start downsample_counter
    downsample_counter=1;
    %start overall counter
    overall_count=1;
    %start the read counter for data storage
    read_count=1;
    while(toc<record_time)
        %wait for a full packet to become available
        while (get(com,'BytesAvailable') < packet_size)
        end
        t_start_loop=toc;
        %read bytes until a start byte appears
        while(fread(com,1)~=2)
        end
        data=[2; fread(com,packet_size-1)];
        start_byte=data(1);  
        data_length=data(2);
        end_byte=data(data_length+3);
        %Check end framing
        if end_byte==3
            %Good end framing, continue checking data
            command_from_PIC=data(3);
            %Check if the data packet is sensor data
            if command_from_PIC==16
                %Yes, the data is sensor data, read and record
                sensor_number=1;
                %Pull the sensor data from the data stream
                for byte_num=4:2:data_length+2
                    sensor_data(sensor_number)=data(byte_num)+256*data(byte_num+1);
                    sensor_number=sensor_number+1;
                end
                %For the first readings, average the readings to determine an
                %appropriate initial value for plot scaling
                if overall_count<=1
                    initial_sensor_data=sensor_data;
                    surf_value=initial_sensor_data;
                end
                if averaging_enabled == 1
                    downsample_data=downsample_data+sensor_data/downsample_count;
                else
                    downsample_data=sensor_data;
                end
                if downsample_counter == downsample_count
                    if show_sample_count==1
                        disp(['Storing sample: ',num2str(read_count)])
                    end
                    %Store the most recent reading
                    if read_count<=data_history_length
                        sensor_data_history(read_count,:)=downsample_data;
                    end
                    %Increment read_count
                    read_count=read_count+1;
                    %Reset downsample_counter
                    downsample_counter=0;
                    %Reset the averaging variable
                    downsample_data=zeros(1,num_sensors);
                end
                downsample_counter=downsample_counter+1;
                overall_count=overall_count+1;
            end
        else
            disp('Bad end framing byte')
        end
        if plot_yes==1
            %if we have time, update plot
            if (get(com,'BytesAvailable') < packet_size*5)
               plot(gca,sensor_data-initial_sensor_data,'*');
               axis(gca,[0 num_sensors -500 2000])
               pause(0.001);
            end
        elseif plot_surf_yes==1
            % initial value filtering if next line is uncommented 
            % initial_value=0.2*value+0.8*initial_value;
            if (get(com,'BytesAvailable') < packet_size*5)
                surf_value=sensor_data-initial_sensor_data;
                if max(surf_value)>max_surf
                    max_surf=max(surf_value);
                    scalingtimer=0;
                else
                    if scalingtimer>20
                        max_surf=max_surf*0.9;
                    end
                    if max_surf<InitialScaling
                        max_surf=InitialScaling;
                    end
                    scalingtimer=scalingtimer+1;
                end
                temp_var=-[0 0 0 0 0 0 0 0;...
                    [0 surf_value([12 11 10 9 8 7]) 0];...
                    [0 surf_value([6 5 4 3 2 1]) 0];...
                    0 0 0 0 0 0 0 0];
                interp_var=interp2(temp_var,2,'cubic');
                size_interp_var=size(interp_var);
                for row_num=1:size_interp_var(1)
                    for column_num=1:size_interp_var(2)
                        row(row_num,column_num)=1.5*(row_num-1)/size_interp_var(1);
                        column(row_num,column_num)=2.5*(column_num-1)/size_interp_var(2);
                    end
                end
                %clf
                surf(gca,row,column,interp_var);
                axis(gca,[-0.25 1.75 0 2.5,...
                    -max_surf InitialScaling*0.5 -max_surf 0]);
                title(gca,'Live View');
                pause(0.01)
            end
        end
    end

    %remove extra preallocated space from data history
    sensor_data_history=sensor_data_history(1:read_count-1,:);
    
    figure
    plot(sensor_data_history-repmat(initial_sensor_data,read_count-1,1));
    title('Data History')
    xlabel('Sample Number')
    ylabel('Counts')

    %Ask the PIC to go into idle mode
    fwrite(com,[2,130,3]);

    %All done, close down any open com ports
    fclose(instrfind);
    delete(instrfind);
end
